/*
* Fabric3
* Copyright (c) 2009 Metaform Systems
*
* Fabric3 is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as
* published by the Free Software Foundation, either version 3 of
* the License, or (at your option) any later version, with the
* following exception:
*
* Linking this software statically or dynamically with other
* modules is making a combined work based on this software.
* Thus, the terms and conditions of the GNU General Public
* License cover the whole combination.
*
* As a special exception, the copyright holders of this software
* give you permission to link this software with independent
* modules to produce an executable, regardless of the license
* terms of these independent modules, and to copy and distribute
* the resulting executable under terms of your choice, provided
* that you also meet, for each linked independent module, the
* terms and conditions of the license of that module. An
* independent module is a module which is not derived from or
* based on this software. If you modify this software, you may
* extend this exception to your version of the software, but
* you are not obligated to do so. If you do not wish to do so,
* delete this exception statement from your version.
*
* Fabric3 is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty
* of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
* See the GNU General Public License for more details.
*
* You should have received a copy of the
* GNU General Public License along with Fabric3.
* If not, see <http://www.gnu.org/licenses/>.
*/
package org.fabric3.pojo.generator;

import java.lang.reflect.Method;

import org.fabric3.model.type.component.ComponentDefinition;
import org.fabric3.model.type.component.Implementation;
import org.fabric3.model.type.java.InjectingComponentType;
import org.fabric3.model.type.java.Signature;
import org.fabric3.pojo.provision.InstanceFactoryDefinition;
import org.fabric3.pojo.provision.PojoComponentDefinition;
import org.fabric3.spi.model.instance.LogicalComponent;

/**
 * @version $Rev: 7285 $ $Date: 2009-07-06 10:45:48 +0200 (Mon, 06 Jul 2009) $
 */
public interface InstanceFactoryGenerationHelper {

    /**
     * Calculates the initialization level (e.g. eager or lazy) for the component.
     *
     * @param definition the component definition.
     * @param type       the component type
     * @return the initialization level
     */
    Integer getInitLevel(ComponentDefinition<?> definition, InjectingComponentType type);

    /**
     * Returns a signature for the method.
     *
     * @param method the method
     * @return the signature
     */
    Signature getSignature(Method method);

    /**
     * Computes injectors for the component
     *
     * @param component          the component
     * @param providerDefinition the instance factor definition for creating component implementation instances
     */
    void processInjectionSites(LogicalComponent<? extends Implementation<InjectingComponentType>> component,
                               InstanceFactoryDefinition providerDefinition);

    /**
     * Set the actual values of the physical properties.
     *
     * @param component the component corresponding to the implementation
     * @param physical  the physical component whose properties should be set
     */
    void processPropertyValues(LogicalComponent<?> component, PojoComponentDefinition physical);
}
